"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stylisPluginCssVariables = stylisPluginCssVariables;

var _weakset = _interopRequireDefault(require("@ungap/weakset"));

var _transform = require("./transform");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
 * This plugin is for the stylis library. It's the CSS compiler used by
 * CSS-in-JS libraries like Emotion.
 *
 * https://github.com/thysultan/stylis.js
 */
var defaultOptions = {
  skipSupportedBrowsers: true
};
/*
 * Generates fallback values for CSS rule declarations that contain CSS var().
 * This plugin parses uses specified fallback values within the var()
 * function. If one is not provided, it will attempt to use the matching
 * variable declared at the :root scope.
 */

function stylisPluginCssVariables(options) {
  if (options === void 0) {
    options = {};
  }

  var _defaultOptions$optio = _objectSpread(_objectSpread({}, defaultOptions), options),
      skipSupportedBrowsers = _defaultOptions$optio.skipSupportedBrowsers;

  var seen = new _weakset.default();
  return function plugin(context, content, selectors, parents, line, column, length, type) {
    // Skip generating CSS variable fallbacks for supported browsers
    if (skipSupportedBrowsers && _utils.isNativeSupport) return; // Borrowed guard implementation from:
    // https://github.com/Andarist/stylis-plugin-extra-scope/blob/master/src/index.js#L15

    if (context !== 2 || type === 107 || seen.has(selectors)) return; // We only need to process the content if a CSS var() is used.

    if (!(0, _utils.hasVariable)(content)) return; // Add to our seen cache, as implemented in stylis-plugin-extra-scope

    seen.add(selectors); // We'll parse the content to match variables to their custom properties (if possible).

    var nextContent = (0, _transform.memoizedTransformContent)(content); // Lastly, we'll provide stylis with our enhanced CSS variable supported content.

    return nextContent;
  };
}