function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } it = o[Symbol.iterator](); return it.next.bind(it); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import memoize from 'memize';
import { getRootPropertyValue, hasVariable, sanitizeParens, VAR_REG_EXP } from './utils';
/**
 * Interprets and retrieves the CSS property and value of a declaration rule.
 *
 * @param {string} declaration A CSS declaration rule to parse.
 * @return {Array<string, ?string>} [prop, value] parsed from the declaration.
 */

function getPropValue(declaration) {
  var hasFallbackValue = false; // Start be separating (and preparing) the prop and value from the declaration.

  var _declaration$replace$ = declaration.replace(/ /g, '').split(/:/),
      _declaration$replace$2 = _slicedToArray(_declaration$replace$, 2),
      prop = _declaration$replace$2[0],
      value = _declaration$replace$2[1]; // Searching for uses of var().


  var matches = value.match(VAR_REG_EXP) || [];

  for (var _iterator = _createForOfIteratorHelperLoose(matches), _step; !(_step = _iterator()).done;) {
    var match = _step.value;
    // Splitting again allows us to traverse through nested vars().
    var entries = match.split('var(').filter(Boolean);

    for (var _iterator2 = _createForOfIteratorHelperLoose(entries), _step2; !(_step2 = _iterator2()).done;) {
      var entry = _step2.value;
      // Removes extra parentheses
      var parsedValue = sanitizeParens(entry);
      /**
       * Splits a CSS variable into it's custom property name and fallback.
       *
       * Before:
       * '--bg, black'
       *
       * After:
       * ['--bg', 'black']
       */

      var _parsedValue$split = parsedValue.split(','),
          _parsedValue$split2 = _slicedToArray(_parsedValue$split, 2),
          customProp = _parsedValue$split2[0],
          customFallback = _parsedValue$split2[1]; // Attempt to get the CSS variable from :root. Otherwise, use the provided fallback.


      var fallback = getRootPropertyValue(customProp) || customFallback;

      if (fallback) {
        hasFallbackValue = true;
        /*
         * If a valid fallback value is discovered, we'll replace it in
         * our value.
         */

        value = value.replace(match, fallback);
      }
    }
  } // We only want to return a value if we're able to locate a fallback value.


  value = hasFallbackValue ? sanitizeParens(value) : undefined;
  return [prop, value];
}
/**
 * Interprets and retrieves the CSS fallback value of a declaration rule.
 *
 * @param {string} declaration A CSS declaration rule to parse.
 * @return {?string} A CSS declaration rule with a fallback (if applicable).
 */


export function getFallbackDeclaration(declaration) {
  if (!hasVariable(declaration)) return undefined;

  var _getPropValue = getPropValue(declaration),
      _getPropValue2 = _slicedToArray(_getPropValue, 2),
      prop = _getPropValue2[0],
      value = _getPropValue2[1];

  return value ? [prop, value].join(':') : undefined;
}
/**
 * Parses the incoming content from stylis to add fallback CSS values for
 * variables.
 *
 * @param {string} content Stylis content to parse.
 * @return {string} The transformed content with CSS variable fallbacks.
 */

export function transformContent(content) {
  /*
   * Attempts to deconstruct the content to retrieve prop/value
   * CSS declaration pairs.
   *
   * Before:
   * 'background-color:var(--bg, black); font-size:14px;'
   *
   * After:
   * ['background-color:var(--bg, black)', ' font-size:14px']
   */
  var declarations = content.split(';').filter(Boolean);
  var didTransform = false;
  /*
   * With the declaration collection, we'll iterate over every declaration
   * to provide fallbacks (if applicable.)
   */

  var parsed = declarations.reduce(function (styles, declaration) {
    // If no CSS variable is used, we return the declaration untouched.
    if (!hasVariable(declaration)) {
      return [].concat(styles, [declaration]);
    } // Retrieve the fallback a CSS variable is used in this declaration.


    var fallback = getFallbackDeclaration(declaration);
    /*
     * Prepend the fallback in our styles set.
     *
     * Before:
     * [
     * 	 ...styles,
     *   'background-color:var(--bg, black);'
     * ]
     *
     * After:
     * [
     * 	 ...styles,
     *   'background:black;',
     *   'background-color:var(--bg, black);'
     * ]
     */

    if (fallback) {
      didTransform = true;
      return [].concat(styles, [fallback, declaration]);
    }

    return [].concat(styles, [declaration]);
  }, []);
  /*
   * We'll rejoin our declarations with a ; separator.
   * Note: We need to add a ; at the end for stylis to interpret correctly.
   */

  var result = parsed.join(';').concat(';'); // We only want to return a value if we're able to locate a fallback value.

  return didTransform ? result : undefined;
}
export var memoizedTransformContent = memoize(transformContent);