var _document, _window, _window$CSS;

export var VAR_REG_EXP = new RegExp(/var\(.*?\)[ ) ]*/, 'g');
var htmlRootNode = (_document = document) == null ? void 0 : _document.documentElement; // Detects native CSS varialble support
// https://github.com/jhildenbiddle/css-vars-ponyfill/blob/master/src/index.js

export var isNativeSupport = (_window = window) == null ? void 0 : (_window$CSS = _window.CSS) == null ? void 0 : _window$CSS.supports('(--a: 0)');
/*
 * Caching the computedStyle instance for document.documentElement.
 * We do this so to prevent additional getComputedStyle calls, which greatly
 * improves performance. We use the .getPropertyValue() method from this
 * reference to retrieve CSS variable values.
 *
 * Although the instance is cached, the values retrieved by .getPropertyValue()
 * are up to date. This is important in cases where global :root variables
 * are updated.
 */

var rootComputedStyles = htmlRootNode && window.getComputedStyle(htmlRootNode);
/**
 * Repeats a character x amount of times.
 * @param {string} char Character to repeat.
 * @param {number} n Number of times to repeat.
 * @return {string} String with repeated characters
 */

function repeat(char, n, a) {
  return (a = []).join(a[n - 1] = char);
}
/**
 * Retrieves the custom CSS variable from the :root selector.
 *
 * @param {string} key The CSS variable property to retrieve.
 * @return {?string} The value of the CSS variable.
 */


export function getRootPropertyValue(key) {
  var _rootStyles, _rootStyles$getProper;

  // We'll attempt to get the CSS variable from :root.
  var rootStyles = rootComputedStyles;

  if (process.env.NODE_ENV === 'test') {
    /*
     * The cached rootComputedStyles does not retrieve the latest values
     * in our environment (JSDOM). In that case, we'll create a fresh
     * instance computedStyles on the root HTML element.
     */
    rootStyles = window.getComputedStyle(document.documentElement);
  }

  return (_rootStyles = rootStyles) == null ? void 0 : (_rootStyles$getProper = _rootStyles.getPropertyValue(key)) == null ? void 0 : _rootStyles$getProper.trim();
}
/**
 * Checks to see if a CSS declaration rule uses var().
 *
 * @param {string} declaration  A CSS declaration rule.
 * @return {boolean} Result of whether declaration contains a CSS variable.
 */

export function hasVariable(declaration) {
  return declaration.includes('var(');
}
/**
 * Appends or trims parens from a value.
 *
 * @param {string} value Value to sanitize.
 * @return {string} The sanitized value
 */

export function sanitizeParens(value) {
  var _value$match, _value$match2;

  var parenStartCount = ((_value$match = value.match(/\(/g)) == null ? void 0 : _value$match.length) || 0;
  var parenEndCount = ((_value$match2 = value.match(/\)/g)) == null ? void 0 : _value$match2.length) || 0;
  var result;

  if (parenStartCount > parenEndCount) {
    // We need to append ) to the end if there are any missing.
    var append = repeat(')', parenStartCount - parenEndCount);
    result = "" + value + append;
  } else {
    // Otherwise, we need to trim the extra parens at the end.
    var trimRegExp = new RegExp("((\\)){" + (parenEndCount - parenStartCount) + "})$", 'gi');
    result = value.replace(trimRegExp, '');
  }

  return result;
}